//========================================================================================
// main.java
// ---------------------------------------------------------------------------------------
//
//----------------------------------------------------------------------------------------
// $Id: //CBT-depot/CompBioToolsApps/JMegaDot/MegaDot.java#6 $
//----------------------------------------------------------------------------------------
//
//========================================================================================
//.......1.........2.........3.........4.........5.........6.........7.........8.........9
//
//
import javax.swing.*;
import javax.swing.event.MouseInputAdapter;
import java.awt.*;
import java.awt.event.*;
import java.util.*;
import java.io.*;

//import com.jgoodies.plaf.FontSizeHints;
//import com.jgoodies.plaf.LookUtils;
//import com.jgoodies.plaf.Options;

public class MegaDot extends JFrame{

  DotPlotEngine mDPE;
  DotPlotPanel  mDPlot;
  
  static {
    // Figure out what platform we are on, OSX, Linux, Windows...
    // extract and load the appropriate native library out of the .jar file. 
    NativeLoader nl = new NativeLoader();
    nl.LoadLibrary("DotPlotEngine");
  }


  /**
   * Configures the UI; tries to set the system look on Mac, 
   * <code>ExtWindowsLookAndFeel</code> on general Windows, and
   * <code>Plastic3DLookAndFeel</code> on Windows XP and all other OS.<p>
   * 
   * The JGoodies Swing Suite's <code>ApplicationStarter</code>,
   * <code>ExtUIManager</code>, and <code>LookChoiceStrategies</code>
   * classes provide a much more fine grained algorithm to choose and
   * restore a look and theme.
   */
  /*
  private void configureUI() {
    UIManager.put(Options.USE_SYSTEM_FONTS_APP_KEY, Boolean.TRUE);
    Options.setGlobalFontSizeHints(FontSizeHints.MIXED);
    Options.setDefaultIconSize(new Dimension(18, 18));
    
    String lafName =
      LookUtils.IS_OS_WINDOWS_XP
      ? Options.getCrossPlatformLookAndFeelClassName()
      : Options.getSystemLookAndFeelClassName();
    
    try {
      UIManager.setLookAndFeel(lafName);
    } catch (Exception e) {
      System.err.println("Can't set look & feel:" + e);
    }
  }
  */

  /**********************************************************
   * Constructor for JMegaDot
   * @param SourceName  Name of single FASTA source file. 
   * @param QueryName   Name of multiple FASTA query file. 
  */
  public MegaDot(String SourceName,String QueryName,int scale,int mersize){
    super("MegaDot");

    //configureUI();

    addWindowListener(new WindowAdapter() {
	public void windowClosing(WindowEvent e) {System.exit(0);}
      });		  

    mDPE = new DotPlotEngine();
    mDPE.SetScale(scale);
    mDPE.SetMerSize(mersize);
    
    System.out.print("Read "+SourceName+"...");
    //mDPE.ReadSource(SourceName);
    mDPE.ReadSourceThinned(SourceName);
    System.out.println("done.");
    System.out.print("Read "+QueryName+"...");
    mDPE.ReadQueries(QueryName);
    System.out.println("done.");

    //mDPE.Compare();
    mDPE.CompareThinned();
    mDPE.ComputeStats();
    mDPE.InitPixels();

    mDPlot = new DotPlotPanel(mDPE);
    
    JPanel plotPanel = new JPanel();
    plotPanel.setLayout(new BorderLayout());
    plotPanel.add(mDPlot,BorderLayout.CENTER);
    // Scrollers 
    //    plotPanel.add(vs,BorderLayout.WEST);
    // plotPanel.add(hs,BorderLayout.SOUTH);

    getContentPane().setLayout(new BorderLayout());
    //getContentPane().add(mTPlot,BorderLayout.CENTER);
    getContentPane().add(plotPanel,BorderLayout.CENTER);
	 
    buildMenu();
    pack();
  }

  private void buildMenu() { // Opens buildMenu method definition 
	 JMenuBar mb = new JMenuBar();
	 JMenu menu = new JMenu("File");
	 JMenuItem item = new JMenuItem("Exit"); 
		
	 item.addActionListener(new ActionListener() { 
	     public void actionPerformed(ActionEvent e) {System.exit(0);}
	   }); // Closes addActionListener method 
				
	 menu.add(item);
	 mb.add(menu);
	 setJMenuBar(mb); 
  }
	 	
  //--------------------------------
  // * main
  //--------------------------------
  //
  public static void main(String argv[]) 
  {
    String sourceName = argv[0];
    String queryName = argv[1];
    System.out.println("source="+sourceName+" query="+queryName);

    int scale = 200;
    if (argv.length > 2){
      Integer I = new Integer(0);
      scale = I.parseInt(argv[2]);
    }


    int mersize = 32;
    if (argv.length > 3){
      Integer I = new Integer(0);
      mersize = I.parseInt(argv[3]);
    }

    //    ControlFrame Controls = new ControlFrame();
    MegaDot JMD= new MegaDot(sourceName,queryName,scale,mersize);				
    JMD.setVisible(true);  
  }
}


/*


TODO:

* Try out hash thinning.... evaluate space compared with regular version...
   - Can we dynamically choose thinning factor to fit the scale of the plot?  When 
     zoomed all the way in, want to recompute the alignment for that region with no sampling. 


* Add slider for repeat cutoff threshold. 


* Space ticks in scale dependent way so that always see a few labels on screen. 
* When zoom in/ zoom out, keep center of display centered.        


* Finish multiple-query feature. 

* Zoom in on mouse selection. 

* Save as JPG. 

* Enter two files in dialog from double clickable application (OS X only). 

* Invert y-axis coordinates. 
* Have an always visible overview with current window highlighted. (Or make it a window).  
* Show sequence on full zoom for pixels. 
* Show alignment for highlighted region. 
* Have a zoom window that shows the current pixel zoomed in a good bit as you mouse over. 

* Enter zoom and x,y offset in a dialog. 
* Add some kind of scale value to slider.  




DONE:

* Fix odd gaps in display. ==> Nothing to fix.  It's N's in the Rat sequence. Huge block from 
   312,800 bp to 698,750 bp.  Looks like it doesn't belong. 


* Show current scale (1 pixel = x bp). 

* Limit number of pixels in mPixels?  
   - Suppose 1 pixel = 16 bp.  Then a 1M x 1M seq at max zoom is:  62.5Kx62.5K pixels = 3.9 *10^15 pixels. 
      - Create an IntensityMap that maps x,y coordinates to intensity. 

* Add reverse complement pixels. 



==================================================================

Thinned version: lastbit vs. lastbit   default scale. 
WindowWidth = 31485
WindowHeight = 25001
*** mIntensityMap.size=55263 mRevIntensityMap.size=33066
WindowWidth=15743
WindowHeight=12501
*** mIntensityMap.size=50788 mRevIntensityMap.size=31025
WindowWidth=7872
WindowHeight=6251
*** mIntensityMap.size=47433 mRevIntensityMap.size=29978
WindowWidth=3936
WindowHeight=3126
*** mIntensityMap.size=43112 mRevIntensityMap.size=28114

Old version:  lastbit vs. lastbit   first few scales:




*/