
import java.awt.*;
import javax.swing.*;

public class Rule extends JComponent {
  public static final int HORIZONTAL = 0;
  public static final int VERTICAL = 1;
  public static final int SIZE = 50;

  public int orientation;
  private int increment;
  
  private int mBasesPerDot;
  private int mTicIncrement = 10000;

  // What is the largest base index we will see? 
  // This is to size the spacing for labels to allow 
  // enough room. 
  private int mMaxBaseNum;

  public void setBasesPerDot(int bpd){
    mBasesPerDot = bpd;
  }

  public Rule(int o,int bpd) {
    mBasesPerDot = bpd;
    System.out.println("mBasesPerDot = "+mBasesPerDot);
    orientation = o;
    setIncrementAndUnits();
  }
  
  private void setIncrementAndUnits() {
    increment = mTicIncrement/mBasesPerDot;
    System.out.println("increment="+increment);
  }
  
  public int getIncrement() {
    return increment;
  }
  
  public void setPreferredHeight(int ph) {
    int maxbase = ph*mBasesPerDot;
    if (maxbase > mMaxBaseNum) maxbase = mMaxBaseNum;

    setPreferredSize(new Dimension(SIZE, ph));
  }

  public void setPreferredWidth(int pw) {
    int maxbase = pw*mBasesPerDot;
    if (maxbase > mMaxBaseNum) maxbase = mMaxBaseNum;

    setPreferredSize(new Dimension(pw, SIZE));
  }

  protected void paintComponent(Graphics g) {
    Rectangle drawHere = g.getClipBounds();

    g.setColor(Color.white);
    g.fillRect(drawHere.x, drawHere.y, drawHere.width, drawHere.height);

    // Do the ruler labels in a small font that's black.
    g.setFont(new Font("SansSerif", Font.PLAIN, 10));
    g.setColor(Color.black);

    // Some vars we need.
    int end = 0;
    int start = 0;
    int tickLength = 0;
    String text = null;

    // Use clipping bounds to calculate first and last tick locations.
    if (orientation == HORIZONTAL) {
      start = (drawHere.x / increment) * increment;
      end = (((drawHere.x + drawHere.width) / increment) + 1)
	* increment;
    } else {
      start = (drawHere.y / increment) * increment;
      end = (((drawHere.y + drawHere.height) / increment) + 1)
	* increment;
    }

    // Make a special case of 0 to display the number
    // within the rule and draw a units label.
    if (start == 0) {
      text = Integer.toString(0) + "bp";
      tickLength = 10;
      if (orientation == HORIZONTAL) {
	g.drawLine(0, SIZE-1, 0, SIZE-tickLength-1);
	g.drawString(text, 2, 21);
      } else {
	g.drawLine(SIZE-1, 0, SIZE-tickLength-1, 0);
	g.drawString(text, 9, 10);
      }
      text = null;
      start = increment;
    }
		 
    String MaxName = Integer.toString(mMaxBaseNum);
    FontMetrics fontMetrics = g.getFontMetrics();
    int MaxNameSize = fontMetrics.stringWidth(MaxName);

    // ticks and labels
    for (int dotIdx = start; dotIdx < end; dotIdx += increment) {
      int bases = mBasesPerDot*dotIdx;
      if (bases % mTicIncrement == 0)  {
	tickLength = 10;
	text = Integer.toString(bases);
      } else {
	tickLength = 7;
	text = null;
      }

      if (tickLength != 0) {
	if (orientation == HORIZONTAL) {
	  g.drawLine(dotIdx, SIZE-1, dotIdx, SIZE-tickLength-1);
	  if (text != null)
	    g.drawString(text, dotIdx-3, 21);
	} else {
	  g.drawLine(SIZE-1, dotIdx, SIZE-tickLength-1, dotIdx);
	  if (text != null)
	    g.drawString(text, 0, dotIdx+3);
	}
      }
    }
  }
}



    /*
    if (orientation == HORIZONTAL){
      System.out.println("HORIZONTAL drawHere.x="+drawHere.x+" drawHere.y="+drawHere.y+
			 " drawHere.width="+drawHere.width+" drawHere.height="+
			 drawHere.height);
      System.out.println("HORIZONTAL start="+start+" end = "+end+" increment="+increment);
    }else{
      System.out.println("VERTICAL drawHere.x="+drawHere.x+" drawHere.y="+drawHere.y+
			 " drawHere.width="+drawHere.width+" drawHere.height="+
			 drawHere.height);

      System.out.println("VERTICAL start="+start+" end = "+end+" increment="+increment);
    }
    */